/****************************************************************************
**
** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** This file is part of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Digia.  For licensing terms and
** conditions see http://qt.digia.com/licensing.  For further information
** use the contact form at http://qt.digia.com/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Digia gives you certain additional
** rights.  These rights are described in the Digia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include "jmediametadataretriever.h"

#include <QtPlatformSupport/private/qjnihelpers_p.h>
#include <qpa/qplatformnativeinterface.h>
#include <qguiapplication.h>

QT_BEGIN_NAMESPACE

static jobject g_activity = 0;

JMediaMetadataRetriever::JMediaMetadataRetriever()
    : QJNIObject("android/media/MediaMetadataRetriever")
{
    if (!g_activity) {
        QPlatformNativeInterface *nativeInterface = QGuiApplication::platformNativeInterface();
        g_activity = static_cast<jobject>(nativeInterface->nativeResourceForIntegration("QtActivity"));
    }
}

JMediaMetadataRetriever::~JMediaMetadataRetriever()
{
}

QString JMediaMetadataRetriever::extractMetadata(MetadataKey key)
{
    QString value;

    QJNILocalRef<jstring> metadata = callObjectMethod<jstring>("extractMetadata",
                                                               "(I)Ljava/lang/String;",
                                                               jint(key));
    if (!metadata.isNull())
        value = qt_convertJString(metadata.object());

    return value;
}

void JMediaMetadataRetriever::release()
{
    callMethod<void>("release");
}

bool JMediaMetadataRetriever::setDataSource(const QUrl &url)
{
    QAttachedJNIEnv env;

    bool loaded = false;

    QJNILocalRef<jstring> string = qt_toJString(url.toString());

    QJNILocalRef<jobject> uri = callStaticObjectMethod<jobject>("android/net/Uri",
                                                                "parse",
                                                                "(Ljava/lang/String;)Landroid/net/Uri;",
                                                                string.object());
    if (env->ExceptionCheck()) {
        env->ExceptionClear();
    } else {
        callMethod<void>("setDataSource",
                         "(Landroid/content/Context;Landroid/net/Uri;)V",
                         g_activity,
                         uri.object());
        if (env->ExceptionCheck())
            env->ExceptionClear();
        else
            loaded = true;
    }

    return loaded;
}

bool JMediaMetadataRetriever::setDataSource(const QString &path)
{
    QAttachedJNIEnv env;

    bool loaded = false;

    callMethod<void>("setDataSource", "(Ljava/lang/String;)V", qt_toJString(path).object());
    if (env->ExceptionCheck())
        env->ExceptionClear();
    else
        loaded = true;

    return loaded;
}

QT_END_NAMESPACE
